#!/usr/bin/env python3
# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2012-2014 Canonical, Ltd.
# Author: Thomi Richards
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

import DistUtilsExtra.auto
from setuptools import find_packages

DistUtilsExtra.auto.setup(
    name='window-mocker',
    version='1.4',
    description=open('README').read(),
    author='Thomi Richards',
    author_email='thomi.richards@canonical.com',
    license='GPLv3',
    packages=find_packages(exclude=('tests',)),
    test_suite='tests',
)
